/*
 * Copyright 2004-2006 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package framework.autoregister;

import java.lang.reflect.Method;

import org.aopalliance.intercept.MethodInterceptor;
import org.seasar.framework.aop.javassist.AspectWeaver;
import org.seasar.framework.container.autoregister.AutoNaming;
import org.seasar.framework.container.autoregister.ClassPattern;
import org.seasar.framework.container.autoregister.DefaultAutoNaming;
import org.seasar.framework.util.ClassUtil;
import org.seasar.framework.util.ClassTraversal.ClassHandler;
import org.springframework.beans.factory.support.RootBeanDefinition;
import org.springframework.beans.factory.xml.BeanDefinitionParserDelegate;
import org.springframework.beans.factory.xml.XmlReaderContext;

public abstract class AbstractBeanAutoRegister extends AbstractAutoRegister
		implements ClassHandler {

	protected static final String CLASS_SUFFIX = ".class";

	private AutoNaming autoNaming = new DefaultAutoNaming();

	private String autowire = "byType";

	private String scope = "singleton";

	public void setAutoNaming(AutoNaming autoNaming) {
		this.autoNaming = autoNaming;
	}

	public void setAutowire(String autowire) {
		this.autowire = autowire;
	}

	public void processClass(final String packageName,
			final String shortClassName) {
		if (isIgnore(packageName, shortClassName)) {
			return;
		}

		for (int i = 0; i < getClassPatternSize(); ++i) {
			final ClassPattern cp = getClassPattern(i);
			if (cp.isAppliedPackageName(packageName)
					&& cp.isAppliedShortClassName(shortClassName)) {
				register(packageName, shortClassName);
			}
		}
	}

	protected void register(final String packageName,
			final String shortClassName) {

		final String className = ClassUtil.concatName(packageName,
				shortClassName);

		Class targetClass = ClassUtil.forName(className);
		AspectWeaver weaver = new AspectWeaver(targetClass, null);
		Method[] methods = targetClass.getDeclaredMethods();
		for (int i = 0; i < methods.length; ++i) {
			weaver.setInterceptors(methods[i], new MethodInterceptor[0]);
		}
		Class enhancedClass = weaver.generateClass();	

		BeanDefinitionParserDelegate delegate = new BeanDefinitionParserDelegate(
				new XmlReaderContext(null, null, null, null, null, null));
		int autowireMode = delegate.getAutowireMode(autowire);

		RootBeanDefinition bd = new RootBeanDefinition();
		bd.setBeanClass(enhancedClass);
		bd.setAutowireMode(autowireMode);
		bd.setScope(scope);

		String beanName;
		if (autoNaming != null) {
			beanName = autoNaming.defineName(packageName, shortClassName);
		} else {
			beanName = className;
		}
		getBeanFactory().registerBeanDefinition(beanName, bd);

	}
}